@@ -17,6 +17,7 @@
 
 static char RCSID[] = "$Id: tclplugin.c,v 1.59 2005/08/11 02:37:55 mooooooo Exp $";
 
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -40,11 +41,14 @@ static char RCSID[] = "$Id: tclplugin.c,v 1.59 2005/08/11 02:37:55 mooooooo Exp
 #include "tclplugin.h"
 #include "printevents.h"
 
+int xchat_plugin_deinit(void);
+void xchat_plugin_get_info(char **name, char **desc, char **version, void **reserved);
+
 static int nexttimerid = 0;
 static int nexttimerindex = 0;
 static timer timers[MAX_TIMERS];
 
-static char VERSION[16];
+static char PVERSION[16];
 
 static int initialized = 0;
 static int reinit_tried = 0;
@@ -2133,7 +2137,7 @@ static void Tcl_Plugin_DeInit()
 
 static void banner()
 {
-    xchat_printf(ph, "Tcl plugin for XChat - Version %s\n", VERSION);
+    xchat_printf(ph, "Tcl plugin for XChat - Version %s\n", PVERSION);
     xchat_print(ph, "Copyright 2002-2005 Daniel P. Stasinski\n");
     xchat_print(ph, "http://www.scriptkitties.com/tclplugin/\n");
 }
@@ -2144,7 +2148,7 @@ int xchat_plugin_init(xchat_plugin * plugin_handle, char **plugin_name, char **p
     HINSTANCE lib;
 #endif
 
-    strncpy(VERSION, &RCSID[19], 5);
+    strncpy(PVERSION, &RCSID[19], 5);
 
     ph = plugin_handle;
 
@@ -2167,7 +2171,7 @@ int xchat_plugin_init(xchat_plugin * plugin_handle, char **plugin_name, char **p
 
     *plugin_name = "Tcl";
     *plugin_desc = "Tcl scripting interface";
-    *plugin_version = VERSION;
+    *plugin_version = PVERSION;
 
     Tcl_Plugin_Init();
 
@@ -2185,7 +2189,7 @@ int xchat_plugin_init(xchat_plugin * plugin_handle, char **plugin_name, char **p
     return 1;                   /* return 1 for success */
 }
 
-int xchat_plugin_deinit()
+int xchat_plugin_deinit(void)
 {
     if (reinit_tried) {
         reinit_tried--;
@@ -2210,10 +2214,10 @@ int xchat_plugin_deinit()
 
 void xchat_plugin_get_info(char **name, char **desc, char **version, void **reserved)
 {
-   strncpy(VERSION, &RCSID[19], 5);
-   *name = "tclplugin";
-   *desc = "Tcl plugin for XChat";
-   *version = VERSION;
+   strncpy(PVERSION, &RCSID[19], 5);
+   *name = _("tclplugin");
+   *desc = _("Tcl plugin for XChat");
+   *version = PVERSION;
    if (reserved)
       *reserved = NULL;
 }
