@@ -44,23 +44,6 @@
 #include "xchatc.h"
 
 
-/* black n white(0/1) are bad colors for nicks, and we'll use color 2 for us */
-/* also light/dark gray (14/15) */
-/* 5,7,8 are all shades of yellow which happen to look dman near the same */
-
-static char rcolors[] = { 19, 20, 22, 24, 25, 26, 27, 28, 29 };
-
-static int
-color_of (char *name)
-{
-	int i = 0, sum = 0;
-
-	while (name[i])
-		sum += name[i++];
-	sum %= sizeof (rcolors) / sizeof (char);
-	return rcolors[sum];
-}
-
 void
 clear_channel (session *sess)
 {
@@ -85,7 +68,7 @@ clear_channel (session *sess)
 	}
 
 	fe_clear_channel (sess);
-	clear_user_list (sess);
+	userlist_clear (sess);
 	fe_set_nonchannel (sess, FALSE);
 	fe_set_title (sess);
 }
@@ -111,13 +94,13 @@ find_session_from_nick (char *nick, server *serv)
 
 	if (serv->front_session)
 	{
-		if (find_name (serv->front_session, nick))
+		if (userlist_find (serv->front_session, nick))
 			return serv->front_session;
 	}
 
 	if (current_sess && current_sess->server == serv)
 	{
-		if (find_name (current_sess, nick))
+		if (userlist_find (current_sess, nick))
 			return current_sess;
 	}
 
@@ -126,7 +109,7 @@ find_session_from_nick (char *nick, server *serv)
 		sess = list->data;
 		if (sess->server == serv)
 		{
-			if (find_name (sess, nick))
+			if (userlist_find (sess, nick))
 				return sess;
 		}
 		list = list->next;
@@ -146,6 +129,24 @@ inbound_open_dialog (server *serv, char *from)
 	return sess;
 }
 
+static void
+inbound_make_idtext (server *serv, char *idtext, int max, int id)
+{
+	idtext[0] = 0;
+	if (serv->have_idmsg)
+	{
+		if (id)
+		{
+			safe_strcpy (idtext, prefs.irc_id_ytext, max);
+		} else
+		{
+			safe_strcpy (idtext, prefs.irc_id_ntext, max);
+		}
+		/* convert codes like %C,%U to the proper ones */
+		check_special_chars (idtext, TRUE);
+	}
+}
+
 void
 inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
 {
@@ -169,6 +170,12 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
 		if (prefs.beepmsg || (sess && sess->beep))
 			sound_beep (sess);
 
+		if (sess && sess->tray)
+			fe_tray_set_icon (2);
+
+		if (prefs.input_flash_priv)
+			fe_flash_window (sess);
+
 		if (ip && ip[0])
 		{
 			if (prefs.logging && sess->logfd != -1 &&
@@ -184,15 +191,7 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
 		return;
 	}
 
-	if (id)
-	{
-		safe_strcpy (idtext, prefs.irc_id_ytext, sizeof (idtext));
-	} else
-	{
-		safe_strcpy (idtext, prefs.irc_id_ntext, sizeof (idtext));
-	}
-	/* convert codes like %C,%U to the proper ones */
-	check_special_chars (idtext, TRUE);
+	inbound_make_idtext (serv, idtext, sizeof (idtext), id);
 
 	sess = find_session_from_nick (from, serv);
 	if (!sess)
@@ -209,6 +208,9 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
 	if (prefs.beepmsg || sess->beep)
 		sound_beep (sess);
 
+	if (prefs.input_flash_priv)
+		fe_flash_window (sess);
+
 	if (sess->type == SESS_DIALOG)
 		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, idtext, NULL, 0);
 	else
@@ -218,7 +220,7 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
 static int
 SearchNick (char *text, char *nicks)
 {
-	char S[300];	/* size of bluestring in xchatprefs */
+	char S[300];	/* size of irc_extra_hilight in xchatprefs */
 	char *n;
 	char *p;
 	char *t;
@@ -227,7 +229,7 @@ SearchNick (char *text, char *nicks)
 	if (nicks == NULL)
 		return 0;
 
-	text = strip_color (text);
+	text = strip_color (text, -1, STRIP_ALL);
 
 	safe_strcpy (S, nicks, sizeof (S));
 	n = strtok (S, ",");
@@ -257,33 +259,54 @@ SearchNick (char *text, char *nicks)
 }
 
 static int
-is_hilight (char *text, session *sess, server *serv)
+FromNick (char *nick, char *nicks)
 {
-	if ((SearchNick (text, serv->nick)) || SearchNick (text, prefs.bluestring))
+	char S[300];	/* size of irc_no_hilight in xchatprefs */
+	char *n;
+	char *t;
+
+	if (nicks == NULL || nicks[0] == 0)
+		return 0;
+
+	safe_strcpy (S, nicks, sizeof (S));
+	n = strtok (S, ",");
+	while (n != NULL)
+	{
+		t = nick;
+		if (nocasestrstr(t, n))
+			return 1;
+		n = strtok (NULL, ",");
+	}
+	return 0;
+}
+
+static int
+is_hilight (char *from, char *text, session *sess, server *serv)
+{
+	if (FromNick(from, prefs.irc_no_hilight))
+		return 0;
+
+	if (SearchNick (text, serv->nick) ||
+		 SearchNick (text, prefs.irc_extra_hilight) ||
+		 FromNick (from, prefs.irc_nick_hilight))
 	{
-#ifdef WIN32
 		if (sess != current_tab)
 			sess->nick_said = TRUE;
 		fe_set_hilight (sess);
-#else
-		if (sess != current_tab)
-		{
-			sess->nick_said = TRUE;
-			fe_set_hilight (sess);
-		}
-#endif
 		return 1;
 	}
 	return 0;
 }
 
 void
-inbound_action (session *sess, char *chan, char *from, char *text, int fromme)
+inbound_action (session *sess, char *chan, char *from, char *text, int fromme, int id)
 {
 	session *def = sess;
 	server *serv = sess->server;
 	int beep = FALSE;
+	struct User *user;
 	int hilight = FALSE;
+	char nickchar[2] = "\000";
 
 	if (!fromme)
 	{
@@ -319,31 +342,40 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme)
 		}
 	}
 
+	user = userlist_find (sess, from);
+	if (user)
+	{
+		nickchar[0] = user->prefix[0];
+		user->lasttalk = time (0);
+	}
+
 	if (!fromme)
 	{
-		hilight = is_hilight (text, sess, serv);
+		hilight = is_hilight (from, text, sess, serv);
 		if (hilight && prefs.beephilight)
 			beep = TRUE;
 
 		if (beep || sess->beep)
 			sound_beep (sess);
 
+		if (sess->tray)
+			fe_tray_set_icon (2);
+
+		/* private action, flash? */
+		if (!is_channel (serv, chan) && prefs.input_flash_priv)
+			fe_flash_window (sess);
+
 		if (hilight)
 		{
-			EMIT_SIGNAL (XP_TE_HCHANACTION, sess, from, text, NULL, NULL, 0);
+			EMIT_SIGNAL (XP_TE_HCHANACTION, sess, from, text, nickchar, NULL, 0);
 			return;
 		}
 	}
 
-	if (prefs.colorednicks)
-	{
-		char tbuf[NICKLEN + 4];
-		snprintf (tbuf, sizeof (tbuf), "\003%d%s", color_of (from), from);
-		EMIT_SIGNAL (XP_TE_CHANACTION, sess, tbuf, text, NULL, NULL, 0);
-	} else
-	{
-		EMIT_SIGNAL (XP_TE_CHANACTION, sess, from, text, NULL, NULL, 0);
-	}
+	if (fromme)
+		EMIT_SIGNAL (XP_TE_UACTION, sess, from, text, nickchar, NULL, 0);
+	else
+		EMIT_SIGNAL (XP_TE_CHANACTION, sess, from, text, nickchar, NULL, 0);
 }
 
 void
@@ -375,7 +407,7 @@ inbound_chanmsg (server *serv, session *sess, char *chan, char *from, char *text
 		sess->new_data = FALSE;
 	}
 
-	user = find_name (sess, from);
+	user = userlist_find (sess, from);
 	if (user)
 	{
 		nickchar[0] = user->prefix[0];
@@ -390,21 +422,18 @@ inbound_chanmsg (server *serv, session *sess, char *chan, char *from, char *text
 		return;
 	}
 
-	if (id)
-	{
-		safe_strcpy (idtext, prefs.irc_id_ytext, sizeof (idtext));
-	} else
-	{
-		safe_strcpy (idtext, prefs.irc_id_ntext, sizeof (idtext));
-	}
-	/* convert codes like %C,%U to the proper ones */
-	check_special_chars (idtext, TRUE);
+	inbound_make_idtext (serv, idtext, sizeof (idtext), id);
 
 	if (sess->type != SESS_DIALOG)
+	{
 		if (prefs.beepchans || sess->beep)
 			sound_beep (sess);
 
-	if (is_hilight (text, sess, serv))
+		if (sess->tray)
+			fe_tray_set_icon (2);
+	}
+
+	if (is_hilight (from, text, sess, serv))
 	{
 		hilight = TRUE;
 		if (prefs.beephilight)
@@ -414,12 +443,6 @@ inbound_chanmsg (server *serv, session *sess, char *chan, char *from, char *text
 		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, idtext, NULL, 0);
 	else if (hilight)
 		EMIT_SIGNAL (XP_TE_HCHANMSG, sess, from, text, nickchar, idtext, 0);
-	else if (prefs.colorednicks)
-	{
-		char tbuf[NICKLEN + 4];
-		snprintf (tbuf, sizeof (tbuf), "\003%d%s", color_of (from), from);
-		EMIT_SIGNAL (XP_TE_CHANMSG, sess, tbuf, text, nickchar, idtext, 0);
-	}
 	else
 		EMIT_SIGNAL (XP_TE_CHANMSG, sess, from, text, nickchar, idtext, 0);
 }
@@ -442,7 +465,7 @@ inbound_newnick (server *serv, char *nick, char *newnick, int quiet)
 		sess = list->data;
 		if (sess->server == serv)
 		{
-			if (change_nick (sess, nick, newnick) || (me && sess->type == SESS_SERVER))
+			if (userlist_change (sess, nick, newnick) || (me && sess->type == SESS_SERVER))
 			{
 				if (!quiet)
 				{
@@ -534,7 +557,7 @@ inbound_ujoin (server *serv, char *chan, char *nick, char *ip)
 	fe_set_channel (sess);
 	fe_set_title (sess);
 	fe_set_nonchannel (sess, TRUE);
-	clear_user_list (sess);
+	userlist_clear (sess);
 
 	if (prefs.logging)
 		log_open (sess);
@@ -609,7 +632,7 @@ inbound_nameslist (server *serv, char *chan, char *names)
 	if (sess->end_of_names)
 	{
 		sess->end_of_names = FALSE;
-		clear_user_list (sess);
+		userlist_clear (sess);
 	}
 
 	while (1)
@@ -619,12 +642,12 @@ inbound_nameslist (server *serv, char *chan, char *names)
 		case 0:
 			name[pos] = 0;
 			if (pos != 0)
-				add_name (sess, name, 0);
+				userlist_add (sess, name, 0);
 			return;
 		case ' ':
 			name[pos] = 0;
 			pos = 0;
-			add_name (sess, name, 0);
+			userlist_add (sess, name, 0);
 			break;
 		default:
 			name[pos] = *names;
@@ -643,7 +666,7 @@ inbound_topic (server *serv, char *chan, char *topic_text)
 
 	if (sess)
 	{
-		new_topic = strip_color (topic_text);
+		new_topic = strip_color (topic_text, -1, STRIP_ALL);
 		set_topic (sess, new_topic);
 		free (new_topic);
 	} else
@@ -661,7 +684,7 @@ inbound_topicnew (server *serv, char *nick, char *chan, char *topic)
 	sess = find_channel (serv, chan);
 	if (sess)
 	{
-		new_topic = strip_color (topic);
+		new_topic = strip_color (topic, -1, STRIP_ALL);
 		set_topic (sess, new_topic);
 		free (new_topic);
 		EMIT_SIGNAL (XP_TE_NEWTOPIC, sess, nick, topic, chan, NULL, 0);
@@ -676,7 +699,7 @@ inbound_join (server *serv, char *chan, char *user, char *ip)
 	{
 		if (!sess->hide_join_part)
 			EMIT_SIGNAL (XP_TE_JOIN, sess, user, chan, ip, NULL, 0);
-		add_name (sess, user, ip);
+		userlist_add (sess, user, ip);
 	}
 }
 
@@ -687,7 +710,7 @@ inbound_kick (server *serv, char *chan, char *user, char *kicker, char *reason)
 	if (sess)
 	{
 		EMIT_SIGNAL (XP_TE_KICK, sess, kicker, user, chan, reason, 0);
-		sub_name (sess, user);
+		userlist_remove (sess, user);
 	}
 }
 
@@ -704,7 +727,7 @@ inbound_part (server *serv, char *chan, char *user, char *ip, char *reason)
 			else
 				EMIT_SIGNAL (XP_TE_PART, sess, user, ip, chan, NULL, 0);
 		}
-		sub_name (sess, user);
+		userlist_remove (sess, user);
 	}
 }
 
@@ -721,42 +744,6 @@ inbound_topictime (server *serv, char *chan, char *nick, time_t stamp)
 	EMIT_SIGNAL (XP_TE_TOPICDATE, sess, chan, nick, tim, NULL, 0);
 }
 
-void
-set_server_name (struct server *serv, char *name)
-{
-	GSList *list = sess_list;
-	session *sess;
-
-	if (name[0] == 0)
-		name = serv->hostname;
-
-	/* strncpy parameters must NOT overlap */
-	if (name != serv->servername)
-	{
-		safe_strcpy (serv->servername, name, sizeof (serv->servername));
-	}
-
-	while (list)
-	{
-		sess = (session *) list->data;
-		if (sess->server == serv)
-			fe_set_title (sess);
-		list = list->next;
-	}
-
-	if (serv->server_session->type == SESS_SERVER)
-	{
-		if (serv->network)
-		{
-			safe_strcpy (serv->server_session->channel, ((ircnet *)serv->network)->name, CHANLEN);
-		} else
-		{
-			safe_strcpy (serv->server_session->channel, name, CHANLEN);
-		}
-		fe_set_channel (serv->server_session);
-	}
-}
-
 void
 inbound_quit (server *serv, char *nick, char *ip, char *reason)
 {
@@ -771,7 +758,7 @@ inbound_quit (server *serv, char *nick, char *ip, char *reason)
 		{
  			if (sess == current_sess)
  				was_on_front_session = TRUE;
-			if (sub_name (sess, nick))
+			if (userlist_remove (sess, nick))
 			{
 				if (!sess->hide_join_part)
 					EMIT_SIGNAL (XP_TE_QUIT, sess, nick, reason, ip, NULL, 0);
@@ -808,8 +795,8 @@ inbound_ping_reply (session *sess, char *timestring, char *from)
 	if (lag)
 	{
 		sess->server->lag_sent = 0;
-		snprintf (outbuf, sizeof (outbuf), "%ld.%ld", dif / 100000, (dif / 10000) % 100);
-		fe_set_lag (sess->server, (int)((float)atof (outbuf)));
+		sess->server->lag = dif / 1000;
+		fe_set_lag (sess->server, dif / 100000);
 		return;
 	}
 
@@ -857,6 +844,12 @@ inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip, int id)
 		sess = find_channel (serv, ptr);
 	}
 
+	if (!sess && ptr[0] == '%')
+	{
+		ptr++;
+		sess = find_channel (serv, ptr);
+	}
+
 	if (!sess && ptr[0] == '+')
 	{
 		ptr++;
@@ -885,7 +878,7 @@ inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip, int id)
 				fe_set_channel (sess);
 				fe_set_title (sess);
 				fe_set_nonchannel (sess, FALSE);
-				clear_user_list (sess);
+				userlist_clear (sess);
 				if (prefs.logging)
 					log_open (sess);
 			}
@@ -894,7 +887,24 @@ inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip, int id)
 				msg += 14;
 		} else
 		{
-			sess = find_session_from_nick (nick, serv);
+											/* paranoia check */
+			if (msg[0] == '[' && (!serv->have_idmsg || id))
+			{
+				/* guess where chanserv meant to post this -sigh- */
+				if (!strcasecmp (nick, "ChanServ") && !find_dialog (serv, nick))
+				{
+					char *dest = strdup (msg + 1);
+					char *end = strchr (dest, ']');
+					if (end)
+					{
+						*end = 0;
+						sess = find_channel (serv, dest);
+					}
+					free (dest);
+				}
+			}
+			if (!sess)
+				sess = find_session_from_nick (nick, serv);
 		}
 		if (!sess)
 		{
@@ -929,7 +939,7 @@ inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip, int id)
 void
 inbound_away (server *serv, char *nick, char *msg)
 {
-	struct away_msg *away = find_away_message (serv, nick);
+	struct away_msg *away = server_away_find_message (serv, nick);
 	session *sess = NULL;
 	GSList *list;
 
@@ -939,13 +949,18 @@ inbound_away (server *serv, char *nick, char *msg)
 			return;
 	} else
 	{
-		save_away_message (serv, nick, msg);
+		server_away_save_message (serv, nick, msg);
 	}
 
-	if (!serv->inside_whois)
-		sess = find_session_from_nick (nick, serv);
-	if (!sess)
-		sess = serv->server_session;
+	if (prefs.irc_whois_front)
+		sess = serv->front_session;
+	else
+	{
+		if (!serv->inside_whois)
+			sess = find_session_from_nick (nick, serv);
+		if (!sess)
+			sess = serv->server_session;
+	}
 
 	EMIT_SIGNAL (XP_TE_WHOIS5, sess, nick, msg, NULL, NULL, 0);
 
@@ -990,12 +1005,17 @@ inbound_nameslist_end (server *serv, char *chan)
 	return FALSE;
 }
 
-static void
+static gboolean
 check_willjoin_channels (server *serv)
 {
 	char *po;
 	session *sess;
 	GSList *list = sess_list;
+	int i = 0;
+
+	/* shouldnt really happen, the io tag is destroyed in server.c */
+	if (!is_server (serv))
+		return FALSE;
 
 	while (list)
 	{
@@ -1013,26 +1033,39 @@ check_willjoin_channels (server *serv)
 				po = strchr (sess->waitchannel, ' ');
 				if (po)
 					*po = 0;
+				i++;
 			}
 		}
 		list = list->next;
 	}
+	serv->joindelay_tag = 0;
+	fe_server_event (serv, FE_SE_LOGGEDIN, i);
+	return FALSE;
 }
 
 void
 inbound_next_nick (session *sess, char *nick)
 {
-	sess->server->nickcount++;
+	char *newnick;
+	server *serv = sess->server;
+	ircnet *net;
 
-	switch (sess->server->nickcount)
+	serv->nickcount++;
+
+	switch (serv->nickcount)
 	{
 	case 2:
-		sess->server->p_change_nick (sess->server, prefs.nick2);
-		EMIT_SIGNAL (XP_TE_NICKCLASH, sess, nick, prefs.nick2, NULL, NULL, 0);
+		newnick = prefs.nick2;
+		net = serv->network;
+		/* use network specific "Second choice"? */
+		if (net && !(net->flags & FLAG_USE_GLOBAL) && net->nick2)
+			newnick = net->nick2;
+		serv->p_change_nick (serv, newnick);
+		EMIT_SIGNAL (XP_TE_NICKCLASH, sess, nick, newnick, NULL, NULL, 0);
 		break;
 
 	case 3:
-		sess->server->p_change_nick (sess->server, prefs.nick3);
+		serv->p_change_nick (serv, prefs.nick3);
 		EMIT_SIGNAL (XP_TE_NICKCLASH, sess, nick, prefs.nick3, NULL, NULL, 0);
 		break;
 
@@ -1080,7 +1113,7 @@ void
 inbound_login_start (session *sess, char *nick, char *servname)
 {
 	inbound_newnick (sess->server, sess->server->nick, nick, TRUE);
-	set_server_name (sess->server, servname);
+	server_set_name (sess->server, servname);
 	if (sess->type == SESS_SERVER && prefs.logging)
 		log_open (sess);
 	/* reset our away status */
@@ -1192,8 +1225,8 @@ inbound_user_info (session *sess, char *chan, char *user, char *host,
 	return 1;
 }
 
-void
-inbound_banlist (session *sess, time_t stamp, char *chan, char *mask, char *banner)
+int
+inbound_banlist (session *sess, time_t stamp, char *chan, char *mask, char *banner, int is_exemption)
 {
 	char *time_str = ctime (&stamp);
 	server *serv = sess->server;
@@ -1204,11 +1237,24 @@ inbound_banlist (session *sess, time_t stamp, char *chan, char *mask, char *bann
 
 	sess = find_channel (serv, chan);
 	if (!sess)
+	{
 		sess = serv->front_session;
+		goto nowindow;
+	}
+
    if (!fe_is_banwindow (sess))
+	{
+nowindow:
+		/* let proto-irc.c do the 'goto def' for exemptions */
+		if (is_exemption)
+			return FALSE;
+
 		EMIT_SIGNAL (XP_TE_BANLIST, sess, chan, mask, banner, time_str, 0);
-	else
-		fe_add_ban_list (sess, mask, banner, time_str);
+		return TRUE;
+	}
+
+	fe_add_ban_list (sess, mask, banner, time_str, is_exemption);
+	return TRUE;
 }
 
 /* execute 1 end-of-motd command */
@@ -1243,11 +1289,16 @@ inbound_login_end (session *sess, char *text)
 
 			/* send nickserv password */
 			if (((ircnet *)serv->network)->nickserv)
-				tcp_sendf (serv, "PRIVMSG NickServ :identify %s\r\n",
-							  ((ircnet *)serv->network)->nickserv);
+				serv->p_ns_identify (serv, ((ircnet *)serv->network)->nickserv);
 		}
 
-		check_willjoin_channels (serv);
+		/* send JOIN now or wait? */
+		if (serv->network && ((ircnet *)serv->network)->nickserv &&
+			 prefs.irc_join_delay)
+			serv->joindelay_tag = fe_timeout_add (prefs.irc_join_delay * 1000,
+															  check_willjoin_channels, serv);
+		else
+			check_willjoin_channels (serv);
 		if (serv->supports_watch)
 			notify_send_watches (serv);
 		serv->end_of_motd = TRUE;
@@ -1262,3 +1313,15 @@ inbound_login_end (session *sess, char *text)
 	EMIT_SIGNAL (XP_TE_MOTD, serv->server_session, text, NULL,
 					 NULL, NULL, 0);
 }
+
+void
+inbound_identified (server *serv)	/* 'MODE +e MYSELF' on freenode */
+{
+	if (serv->joindelay_tag)
+	{
+		/* stop waiting, just auto JOIN now */
+		fe_timeout_remove (serv->joindelay_tag);
+		serv->joindelay_tag = 0;
+		check_willjoin_channels (serv);
+	}
+}
