@@ -16,6 +16,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include <config.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
@@ -27,6 +28,8 @@
 #include "outbound.h"
 #include "cfgfiles.h"
 #include "ignore.h"
+#include "server.h"
+#include "servlist.h"
 #include "modes.h"
 #include "notify.h"
 #include "text.h"
@@ -105,21 +108,6 @@ static GSList *hook_list = NULL;
 extern const struct prefs vars[];	/* cfgfiles.c */
 
 
-/* 31 bit string hash function */
-
-static guint32
-str_hash (const char *key)
-{
-	const char *p = key;
-	guint32 h = *p;
-
-	if (h)
-		for (p += 1; *p != '\0'; p++)
-			h = (h << 5) - h + *p;
-
-	return h;
-}
-
 /* unload a plugin and remove it from our linked list */
 
 static int
@@ -162,6 +150,15 @@ plugin_free (xchat_plugin *pl, int do_deinit, int allow_refuse)
 #endif
 
 xit:
+	if (pl->free_strings)
+	{
+		if (pl->name)
+			free (pl->name);
+		if (pl->desc)
+			free (pl->desc);
+		if (pl->version)
+			free (pl->version);
+	}
 	if (pl->filename)
 		free ((char *)pl->filename);
 	free (pl);
@@ -178,7 +175,7 @@ xit:
 static xchat_plugin *
 plugin_list_add (xchat_context *ctx, char *filename, const char *name,
 					  const char *desc, const char *version, void *handle,
-					  void *deinit_func, int fake)
+					  void *deinit_func, int fake, int free_strings)
 {
 	xchat_plugin *pl;
 
@@ -191,6 +188,7 @@ plugin_list_add (xchat_context *ctx, char *filename, const char *name,
 	pl->version = (char *)version;
 	pl->deinit_callback = deinit_func;
 	pl->fake = fake;
+	pl->free_strings = free_strings;	/* free() name,desc,version? */
 
 	plugin_list = g_slist_prepend (plugin_list, pl);
 
@@ -225,7 +223,7 @@ plugin_add (session *sess, char *filename, void *handle, void *init_func,
 		file = strdup (filename);
 
 	pl = plugin_list_add (sess, file, file, NULL, NULL, handle, deinit_func,
-								 fake);
+								 fake, FALSE);
 
 	if (!fake)
 	{
@@ -418,6 +416,11 @@ plugin_auto_load_cb (char *filename)
 {
 	char *pMsg;
 
+#ifndef WIN32	/* black listed */
+	if (!strcmp (file_part (filename), "dbus.so"))
+		return;
+#endif
+
 	pMsg = plugin_load (ps, filename, NULL);
 	if (pMsg)
 	{
@@ -613,7 +616,7 @@ plugin_timeout_cb (xchat_hook *hook)
 	ret = ((xchat_timer_cb *)hook->callback) (hook->userdata);
 
 	/* the callback might have already unhooked it! */
-	if (!g_slist_find (hook_list, hook))
+	if (!g_slist_find (hook_list, hook) || hook->type == HOOK_DELETED)
 		return 0;
 
 	if (ret == 0)
@@ -710,26 +713,31 @@ plugin_command_list(GList *tmp_list)
 	return tmp_list;
 }
 
+void
+plugin_command_foreach (session *sess, void *userdata,
+			void (*cb) (session *sess, void *userdata, char *name, char *help))
+{
+	GSList *list;
+	xchat_hook *hook;
+
+	list = hook_list;
+	while (list)
+	{
+		hook = list->data;
+		if (hook->type == HOOK_COMMAND && hook->name[0])
+		{
+			cb (sess, userdata, hook->name, hook->help_text);
+		}
+		list = list->next;
+	}
+}
+
 int
 plugin_show_help (session *sess, char *cmd)
 {
 	GSList *list;
 	xchat_hook *hook;
 
-	/* show all help commands */
-	if (cmd == NULL)
-	{
-		list = hook_list;
-		while (list)
-		{
-			hook = list->data;
-			if (hook->type == HOOK_COMMAND)
-				PrintText (sess, hook->name);
-			list = list->next;
-		}
-		return 1;
-	}
-
 	list = plugin_hook_find (hook_list, HOOK_COMMAND, cmd);
 	if (list)
 	{
@@ -752,7 +760,7 @@ void *
 xchat_unhook (xchat_plugin *ph, xchat_hook *hook)
 {
 	/* perl.c trips this */
-	if (hook->type == HOOK_DELETED || !g_slist_find (hook_list, hook))
+	if (!g_slist_find (hook_list, hook) || hook->type == HOOK_DELETED)
 		return NULL;
 
 	if (hook->type == HOOK_TIMER && hook->tag != 0)
@@ -843,13 +851,19 @@ xchat_printf (xchat_plugin *ph, const char *format, ...)
 void
 xchat_command (xchat_plugin *ph, const char *command)
 {
+	char *conv;
+	int len = -1;
+
 	if (!is_session (ph->context))
 	{
 		DEBUG(PrintTextf(0, "%s\txchat_command called without a valid context.\n", ph->name));
 		return;
 	}
 
+	/* scripts/plugins continue to send non-UTF8... *sigh* */
+	conv = text_validate ((char **)&command, &len);
 	handle_command (ph->context, (char *)command, FALSE);
+	g_free (conv);
 }
 
 void
@@ -892,7 +906,7 @@ xchat_set_context (xchat_plugin *ph, xchat_context *context)
 xchat_context *
 xchat_find_context (xchat_plugin *ph, const char *servname, const char *channel)
 {
-	GSList *slist, *clist;
+	GSList *slist, *clist, *sessions = NULL;
 	server *serv;
 	session *sess;
 	char *netname;
@@ -904,7 +918,7 @@ xchat_find_context (xchat_plugin *ph, const char *servname, const char *channel)
 	while (slist)
 	{
 		serv = slist->data;
-		netname = get_network (serv->front_session, TRUE);
+		netname = server_get_network (serv, TRUE);
 
 		if (servname == NULL ||
 			 rfc_casecmp (servname, serv->servername) == 0 ||
@@ -921,7 +935,16 @@ xchat_find_context (xchat_plugin *ph, const char *servname, const char *channel)
 				if (sess->server == serv)
 				{
 					if (rfc_casecmp (channel, sess->channel) == 0)
-						return sess;
+					{
+						if (sess->server == ph->context->server)
+						{
+							g_slist_free (sessions);
+							return sess;
+						} else
+						{
+							sessions = g_slist_prepend (sessions, sess);
+						}
+					}
 				}
 				clist = clist->next;
 			}
@@ -929,6 +952,14 @@ xchat_find_context (xchat_plugin *ph, const char *servname, const char *channel)
 		slist = slist->next;
 	}
 
+	if (sessions)
+	{
+		sessions = g_slist_reverse (sessions);
+		sess = sessions->data;
+		g_slist_free (sessions);
+		return sess;
+	}
+
 	return NULL;
 }
 
@@ -936,6 +967,48 @@ const char *
 xchat_get_info (xchat_plugin *ph, const char *id)
 {
 	session *sess;
+	guint32 hash;
+
+	/*                 1234567890 */
+	if (!strncmp (id, "event_text", 10))
+	{
+		char *e = (char *)id + 10;
+		if (*e == ' ') e++;	/* 2.8.0 only worked without a space */
+		return text_find_format_string (e);
+	}
+
+	hash = str_hash (id);
+	/* do the session independant ones first */
+	switch (hash)
+	{
+	case 0x325acab5:	/* libdirfs */
+		return XCHATLIBDIR;
+
+	case 0x14f51cd8: /* version */
+		return PACKAGE_VERSION;
+
+	case 0xdd9b1abd:	/* xchatdir */
+		return get_xdir_utf8 ();
+
+	case 0xe33f6c4a:	/* xchatdirfs */
+		return get_xdir_fs ();
+	}
+
+	/* do the session independant ones first */
+	switch (str_hash (id))
+	{
+	case 0x325acab5:	/* libdirfs */
+		return XCHATLIBDIR;
+
+	case 0x14f51cd8: /* version */
+		return PACKAGE_VERSION;
+
+	case 0xdd9b1abd:	/* xchatdir */
+		return get_xdir_utf8 ();
+
+	case 0xe33f6c4a:	/* xchatdirfs */
+		return get_xdir_fs ();
+	}
 
 	sess = ph->context;
 	if (!is_session (sess))
@@ -944,7 +1017,7 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 		return NULL;
 	}
 
-	switch (str_hash (id))
+	switch (hash)
 	{
 	case 0x2de2ee: /* away */
 		if (sess->server->is_away)
@@ -972,15 +1045,20 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x1c0e99c1: /* inputbox */
 		return fe_get_inputbox_contents (sess);
 
-	case 0x325acab5:	/* libdirfs */
-		return XCHATLIBDIR;
+	case 0x633fb30:	/* modes */
+		return sess->current_modes;
 
 	case 0x6de15a2e:	/* network */
-		return get_network (sess, FALSE);
+		return server_get_network (sess->server, FALSE);
 
 	case 0x339763: /* nick */
 		return sess->server->nick;
 
+	case 0x438fdf9: /* nickserv */
+		if (sess->server->network)
+			return ((ircnet *)sess->server->network)->nickserv;
+		return NULL;
+
 	case 0xca022f43: /* server */
 		if (!sess->server->connected)
 			return NULL;
@@ -989,8 +1067,8 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x696cd2f: /* topic */
 		return sess->topic;
 
-	case 0x14f51cd8: /* version */
-		return VERSION;
+	case 0x506d600b: /* native win_ptr */
+		return fe_gui_info_ptr (sess, 0);
 
 	case 0x6d3431b5: /* win_status */
 		switch (fe_gui_info (sess, 0))	/* check window status */
@@ -1000,12 +1078,6 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 		case 2: return "hidden";
 		}
 		return NULL;
-
-	case 0xdd9b1abd:	/* xchatdir */
-		return get_xdir_utf8 ();
-
-	case 0xe33f6c4a:	/* xchatdirfs */
-		return get_xdir_fs ();
 	}
 
 	return NULL;
@@ -1017,12 +1089,17 @@ xchat_get_prefs (xchat_plugin *ph, const char *name, const char **string, int *i
 	int i = 0;
 
 	/* some special run-time info (not really prefs, but may aswell throw it in here) */
-	if (!strcmp (name, "state_cursor"))
+	switch (str_hash (name))
 	{
-		*integer = fe_get_inputbox_cursor (ph->context);
-		return 2;
-	}
+		case 0xf82136c4: /* state_cursor */
+			*integer = fe_get_inputbox_cursor (ph->context);
+			return 2;
 
+		case 0xd1b: /* id */
+			*integer = ph->context->server->id;
+			return 2;
+	}
+	
 	do
 	{
 		if (!strcasecmp (name, vars[i].name))
@@ -1089,6 +1166,7 @@ xchat_list_get (xchat_plugin *ph, const char *name)
 		{
 			list->type = LIST_USERS;
 			list->head = list->next = userlist_flat_list (ph->context);
+			fe_userlist_set_selected (ph->context);
 			break;
 		}	/* fall through */
 
@@ -1136,12 +1214,12 @@ xchat_list_fields (xchat_plugin *ph, const char *name)
 	static const char * const dcc_fields[] =
 	{
 		"iaddress32","icps",		"sdestfile","sfile",		"snick",	"iport",
-		"ipos",		"iresume",	"isize",		"istatus", 	"itype",		NULL
+		"ipos", "iresume", "isize", "isizehigh", "istatus", "itype", NULL
 	};
 	static const char * const channels_fields[] =
 	{
-		"schannel",	"schantypes", "pcontext", "iflags", "iid", "imaxmodes",
-		"snetwork", "snickmodes", "snickprefixes", "sserver", "itype", "iusers",
+		"schannel",	"schantypes", "pcontext", "iflags", "iid", "ilag", "imaxmodes",
+		"snetwork", "snickmodes", "snickprefixes", "iqueue", "sserver", "itype", "iusers",
 		NULL
 	};
 	static const char * const ignore_fields[] =
@@ -1150,11 +1228,11 @@ xchat_list_fields (xchat_plugin *ph, const char *name)
 	};
 	static const char * const notify_fields[] =
 	{
-		"iflags", "snick", "toff", "ton", "tseen", NULL
+		"iflags", "snetworks", "snick", "toff", "ton", "tseen", NULL
 	};
 	static const char * const users_fields[] =
 	{
-		"iaway", "shost", "tlasttalk", "snick", "sprefix", NULL
+		"iaway", "shost", "tlasttalk", "snick", "sprefix", "iselected", NULL
 	};
 	static const char * const list_of_lists[] =
 	{
@@ -1218,9 +1296,17 @@ const char *
 xchat_list_str (xchat_plugin *ph, xchat_list *xlist, const char *name)
 {
 	guint32 hash = str_hash (name);
-	gpointer data = xlist->pos->data;
+	gpointer data = ph->context;
+	int type = LIST_CHANNELS;
 
-	switch (xlist->type)
+	/* a NULL xlist is a shortcut to current "channels" context */
+	if (xlist)
+	{
+		data = xlist->pos->data;
+		type = xlist->type;
+	}
+
+	switch (type)
 	{
 	case LIST_CHANNELS:
 		switch (hash)
@@ -1232,7 +1318,7 @@ xchat_list_str (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		case 0x38b735af: /* context */
 			return data;	/* this is a session * */
 		case 0x6de15a2e: /* network */
-			return get_network ((session *)data, FALSE);
+			return server_get_network (((session *)data)->server, FALSE);
 		case 0x8455e723: /* nickprefixes */
 			return ((session *)data)->server->nick_prefixes;
 		case 0x829689ad: /* nickmodes */
@@ -1265,6 +1351,8 @@ xchat_list_str (xchat_plugin *ph, xchat_list *xlist, const char *name)
 	case LIST_NOTIFY:
 		switch (hash)
 		{
+		case 0x4e49ec05:	/* networks */
+			return ((struct notify *)data)->networks;
 		case 0x339763: /* nick */
 			return ((struct notify *)data)->name;
 		}
@@ -1290,10 +1378,17 @@ int
 xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 {
 	guint32 hash = str_hash (name);
-	gpointer data = xlist->pos->data;
-	int tmp;
+	gpointer data = ph->context;
+	int tmp, type = LIST_CHANNELS;
 
-	switch (xlist->type)
+	/* a NULL xlist is a shortcut to current "channels" context */
+	if (xlist)
+	{
+		data = xlist->pos->data;
+		type = xlist->type;
+	}
+
+	switch (type)
 	{
 	case LIST_DCC:
 		switch (hash)
@@ -1309,7 +1404,9 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		case 0xc84dc82d: /* resume */
 			return ((struct DCC *)data)->resumable;
 		case 0x35e001: /* size */
-			return ((struct DCC *)data)->size;
+			return ((struct DCC *)data)->size & 0xffffffff;
+		case 0x3284d523: /* sizehigh */
+			return (((struct DCC *)data)->size >> 32) & 0xffffffff;
 		case 0xcacdcff2: /* status */
 			return ((struct DCC *)data)->dccstat;
 		case 0x368f3a: /* type */
@@ -1331,7 +1428,17 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		case 0xd1b:	/* id */
 			return ((struct session *)data)->server->id;
 		case 0x5cfee87:	/* flags */
-			tmp = ((struct session *)data)->server->have_whox;   /* bit 4 */
+			tmp = ((struct session *)data)->tray;            /* bit 9 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->beep;               /* 8 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->color_paste;        /* 7 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->hide_join_part;     /* 6 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->server->have_idmsg; /* 5 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->server->have_whox;  /* 4 */
 			tmp <<= 1;
 			tmp |= ((struct session *)data)->server->end_of_motd;/* 3 */
 			tmp <<= 1;
@@ -1341,8 +1448,12 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 			tmp <<= 1;
 			tmp |= ((struct session *)data)->server->connected;  /* 0 */
 			return tmp;
+		case 0x1a192: /* lag */
+			return ((struct session *)data)->server->lag;
 		case 0x1916144c: /* maxmodes */
 			return ((struct session *)data)->server->modes_per_line;
+		case 0x66f1911: /* queue */
+			return ((struct session *)data)->server->sendq_len;
 		case 0x368f3a:	/* type */
 			return ((struct session *)data)->type;
 		case 0x6a68e08: /* users */
@@ -1364,6 +1475,8 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		{
 		case 0x2de2ee:	/* away */
 			return ((struct User *)data)->away;
+		case 0x4705f29b: /* selected */
+			return ((struct User *)data)->selected;
 		}
 		break;
 
@@ -1378,8 +1491,8 @@ xchat_plugingui_add (xchat_plugin *ph, const char *filename,
 							const char *version, char *reserved)
 {
 #ifdef USE_PLUGIN
-	ph = plugin_list_add (NULL, strdup (filename), name, desc, version, NULL,
-								 NULL, TRUE);
+	ph = plugin_list_add (NULL, strdup (filename), strdup (name), strdup (desc),
+								 strdup (version), NULL, NULL, TRUE, TRUE);
 	fe_pluginlist_update ();
 #endif
 
@@ -1398,10 +1511,11 @@ int
 xchat_emit_print (xchat_plugin *ph, const char *event_name, ...)
 {
 	va_list args;
+	/* currently only 4 because no events use more than 4.
+		This can be easily expanded without breaking the API. */
 	char *argv[4] = {NULL, NULL, NULL, NULL};
 	int i = 0;
 
-	memset (&argv, 0, sizeof (argv));
 	va_start (args, event_name);
 	while (1)
 	{
@@ -1439,7 +1553,7 @@ xchat_send_modes (xchat_plugin *ph, const char **targets, int ntargets, int mode
 char *
 xchat_strip (xchat_plugin *ph, const char *str, int len, int flags)
 {
-	return strip_color ((char *)str, len, (flags & 1), (flags & 2));
+	return strip_color ((char *)str, len, flags);
 }
 
 void
