@@ -14,8 +14,15 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ *
+ * MS Proxy (ISA server) support is (c) 2006 Pavel Fedin <sonic_amiga@rambler.ru>
+ * based on Dante source code
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+ *      Inferno Nettverk A/S, Norway.  All rights reserved.
  */
 
+/*#define DEBUG_MSPROXY*/
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -55,6 +62,10 @@
 #include "ssl.h"
 #endif
 
+#ifdef USE_MSPROXY
+#include "msproxy.h"
+#endif
+
 #ifdef WIN32
 #include "identd.c"
 #endif
@@ -75,18 +86,16 @@ static void server_connect (server *serv, char *hostname, int port, int no_login
 
 
 /* actually send to the socket. This might do a character translation or
-   send via SSL */
+   send via SSL. server/dcc both use this function. */
 
-static int
-tcp_send_real (server *serv, char *buf, int len)
+int
+tcp_send_real (void *ssl, int sok, char *encoding, int using_irc, char *buf, int len)
 {
 	int ret;
 	char *locale;
 	gsize loc_len;
 
-	fe_add_rawlog (serv, buf, len, TRUE);
-
-	if (serv->encoding == NULL)	/* system */
+	if (encoding == NULL)	/* system */
 	{
 		locale = NULL;
 		if (!prefs.utf8_locale)
@@ -99,37 +108,51 @@ tcp_send_real (server *serv, char *buf, int len)
 		}
 	} else
 	{
-		locale = g_convert_with_fallback (buf, len, serv->encoding, "UTF-8",
-													 "?", 0, &loc_len, 0);
+		if (using_irc)	/* using "IRC" encoding (CP1252/UTF-8 hybrid) */
+			/* if all chars fit inside CP1252, use that. Otherwise this
+			   returns NULL and we send UTF-8. */
+			locale = g_convert (buf, len, "CP1252", "UTF-8", 0, &loc_len, 0);
+		else
+			locale = g_convert_with_fallback (buf, len, encoding, "UTF-8",
+														 "?", 0, &loc_len, 0);
 	}
 
 	if (locale)
 	{
 		len = loc_len;
 #ifdef USE_OPENSSL
-		if (!serv->ssl)
-			ret = send (serv->sok, locale, len, 0);
+		if (!ssl)
+			ret = send (sok, locale, len, 0);
 		else
-			ret = _SSL_send (serv->ssl, locale, len);
+			ret = _SSL_send (ssl, locale, len);
 #else
-		ret = send (serv->sok, locale, len, 0);
+		ret = send (sok, locale, len, 0);
 #endif
 		g_free (locale);
 	} else
 	{
 #ifdef USE_OPENSSL
-		if (!serv->ssl)
-			ret = send (serv->sok, buf, len, 0);
+		if (!ssl)
+			ret = send (sok, buf, len, 0);
 		else
-			ret = _SSL_send (serv->ssl, buf, len);
+			ret = _SSL_send (ssl, buf, len);
 #else
-		ret = send (serv->sok, buf, len, 0);
+		ret = send (sok, buf, len, 0);
 #endif
 	}
 
 	return ret;
 }
 
+static int
+server_send_real (server *serv, char *buf, int len)
+{
+	fe_add_rawlog (serv, buf, len, TRUE);
+
+	return tcp_send_real (serv->ssl, serv->sok, serv->encoding, serv->using_irc,
+								 buf, len);
+}
+
 /* new throttling system, uses the same method as the Undernet
    ircu2.10 server; under test, a 200-line paste didn't flood
    off the client */
@@ -146,8 +169,8 @@ tcp_send_queue (server *serv)
 	if (!is_server (serv))
 		return 0;
 
-	/* first try priority 1 entries */
-	pri = 1;
+	/* try priority 2,1,0 */
+	pri = 2;
 	while (pri >= 0)
 	{
 		list = serv->outbound_queue;
@@ -176,7 +199,7 @@ tcp_send_queue (server *serv)
 				serv->prev_now = now;
 				fe_set_throttle (serv);
 
-				tcp_send_real (serv, buf, len);
+				server_send_real (serv, buf, len);
 
 				buf--;
 				serv->outbound_queue = g_slist_remove (serv->outbound_queue, buf);
@@ -200,17 +223,29 @@ tcp_send_len (server *serv, char *buf, int len)
 	int noqueue = !serv->outbound_queue;
 
 	if (!prefs.throttle)
-		return tcp_send_real (serv, buf, len);
+		return server_send_real (serv, buf, len);
 
 	dbuf = malloc (len + 2);	/* first byte is the priority */
-	dbuf[0] = 1;	/* pri 1 for most things */
+	dbuf[0] = 2;	/* pri 2 for most things */
 	memcpy (dbuf + 1, buf, len);
 	dbuf[len + 1] = 0;
 
 	/* privmsg and notice get a lower priority */
 	if (strncasecmp (dbuf + 1, "PRIVMSG", 7) == 0 ||
 		 strncasecmp (dbuf + 1, "NOTICE", 6) == 0)
-		dbuf[0] = 0;
+	{
+		dbuf[0] = 1;
+	}
+	else
+	{
+		/* WHO/MODE get the lowest priority */
+		if (strncasecmp (dbuf + 1, "WHO ", 4) == 0 ||
+		/* but only MODE queries, not changes */
+			(strncasecmp (dbuf + 1, "MODE", 4) == 0 &&
+			 strchr (dbuf, '-') == NULL &&
+			 strchr (dbuf, '+') == NULL))
+			dbuf[0] = 0;
+	}
 
 	serv->outbound_queue = g_slist_append (serv->outbound_queue, dbuf);
 	serv->sendq_len += len; /* tcp_send_queue uses strlen */
@@ -269,8 +304,9 @@ server_inline (server *serv, char *line, int len)
 	char *utf_line_allocated = NULL;
 
 	/* Checks whether we're set to use UTF-8 charset */
-	if ((serv->encoding == NULL && prefs.utf8_locale) ||
-	    (serv->encoding != NULL &&
+	if (serv->using_irc ||				/* 1. using CP1252/UTF-8 Hybrid */
+		(serv->encoding == NULL && prefs.utf8_locale) || /* OR 2. using system default->UTF-8 */
+	    (serv->encoding != NULL &&				/* OR 3. explicitly set to UTF-8 */
 		 (strcasecmp (serv->encoding, "UTF8") == 0 ||
 		  strcasecmp (serv->encoding, "UTF-8") == 0)))
 	{
@@ -371,7 +407,6 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 	int sok = serv->sok;
 	int error, i, len;
 	char lbuf[2050];
-	char *temp;
 
 	while (1)
 	{
@@ -423,15 +458,7 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 
 			case '\n':
 				serv->linebuf[serv->pos] = 0;
-				if (prefs.stripcolor)
-				{
-					temp = strip_color (serv->linebuf, -1, 1, 1);
-					server_inline (serv, temp, strlen (temp));
-					free (temp);
-				} else
-				{
-					server_inline (serv, serv->linebuf, serv->pos);
-				}
+				server_inline (serv, serv->linebuf, serv->pos);
 				serv->pos = 0;
 				break;
 
@@ -543,6 +570,7 @@ server_stopconnecting (server * serv)
 	fe_progressbar_end (serv);
 
 	serv->connecting = FALSE;
+	fe_server_event (serv, FE_SE_DISCONNECT, 0);
 }
 
 #ifdef USE_OPENSSL
@@ -791,6 +819,13 @@ auto_reconnect (server *serv, int send_quit, int err)
 #endif
 		serv->reconnect_away = serv->is_away;
 
+	/* is this server in a reconnect delay? remove it! */
+	if (serv->recondelay_tag)
+	{
+		fe_timeout_remove (serv->recondelay_tag);
+		serv->recondelay_tag = 0;
+	}
+
 	serv->recondelay_tag = fe_timeout_add (del, timeout_auto_reconnect, serv);
 	fe_server_event (serv, FE_SE_RECONDELAY, del);
 }
@@ -876,6 +911,7 @@ server_read_child (GIOChannel *source, GIOCondition condition, server *serv)
 	char outbuf[512];
 	char host[100];
 	char ip[100];
+	char *p;
 
 	waitline2 (source, tbuf, sizeof tbuf);
 
@@ -888,9 +924,13 @@ server_read_child (GIOChannel *source, GIOCondition condition, server *serv)
 	case '1':						  /* unknown host */
 		server_stopconnecting (serv);
 		closesocket (serv->sok4);
+		if (serv->proxy_sok4 != -1)
+			closesocket (serv->proxy_sok4);
 #ifdef USE_IPV6
 		if (serv->sok6 != -1)
 			closesocket (serv->sok6);
+		if (serv->proxy_sok6 != -1)
+			closesocket (serv->proxy_sok6);
 #endif
 		EMIT_SIGNAL (XP_TE_UKNHOST, sess, NULL, NULL, NULL, NULL, 0);
 		if (!servlist_cycle (serv))
@@ -901,9 +941,13 @@ server_read_child (GIOChannel *source, GIOCondition condition, server *serv)
 		waitline2 (source, tbuf, sizeof tbuf);
 		server_stopconnecting (serv);
 		closesocket (serv->sok4);
+		if (serv->proxy_sok4 != -1)
+			closesocket (serv->proxy_sok4);
 #ifdef USE_IPV6
 		if (serv->sok6 != -1)
 			closesocket (serv->sok6);
+		if (serv->proxy_sok6 != -1)
+			closesocket (serv->proxy_sok6);
 #endif
 		EMIT_SIGNAL (XP_TE_CONNFAIL, sess, errorstring (atoi (tbuf)), NULL,
 						 NULL, NULL, 0);
@@ -939,13 +983,36 @@ server_read_child (GIOChannel *source, GIOCondition condition, server *serv)
 		break;
 	case '4':						  /* success */
 		waitline2 (source, tbuf, sizeof (tbuf));
+#ifdef USE_MSPROXY
+		serv->sok = strtol (tbuf, &p, 10);
+		if (*p++ == ' ')
+		{
+			serv->proxy_sok = strtol (p, &p, 10);
+			serv->msp_state.clientid = strtol (++p, &p, 10);
+			serv->msp_state.serverid = strtol (++p, &p, 10);
+			serv->msp_state.seq_sent = atoi (++p);
+		} else
+			serv->proxy_sok = -1;
+#ifdef DEBUG_MSPROXY
+		printf ("Parent got main socket: %d, proxy socket: %d\n", serv->sok, serv->proxy_sok);
+		printf ("Client ID 0x%08x server ID 0x%08x seq_sent %d\n", serv->msp_state.clientid, serv->msp_state.serverid, serv->msp_state.seq_sent);
+#endif
+#else
 		serv->sok = atoi (tbuf);
+#endif
 #ifdef USE_IPV6
 		/* close the one we didn't end up using */
 		if (serv->sok == serv->sok4)
 			closesocket (serv->sok6);
 		else
 			closesocket (serv->sok4);
+		if (serv->proxy_sok != -1)
+		{
+			if (serv->proxy_sok == serv->proxy_sok4)
+				closesocket (serv->proxy_sok6);
+			else
+				closesocket (serv->proxy_sok4);
+		}
 #endif
 		server_connect_success (serv);
 		break;
@@ -1004,14 +1071,20 @@ server_cleanup (server * serv)
 	{
 		server_stopconnecting (serv);
 		closesocket (serv->sok4);
+		if (serv->proxy_sok4 != -1)
+			closesocket (serv->proxy_sok4);
 		if (serv->sok6 != -1)
 			closesocket (serv->sok6);
+		if (serv->proxy_sok6 != -1)
+			closesocket (serv->proxy_sok6);
 		return 1;
 	}
 
 	if (serv->connected)
 	{
 		close_socket (serv->sok);
+		if (serv->proxy_sok)
+			close_socket (serv->proxy_sok);
 		serv->connected = FALSE;
 		serv->end_of_motd = FALSE;
 		return 2;
@@ -1034,6 +1107,7 @@ server_disconnect (session * sess, int sendquit, int err)
 	server *serv = sess->server;
 	GSList *list;
 	char tbuf[64];
+	gboolean shutup = FALSE;
 
 	/* send our QUIT reason */
 	if (sendquit && serv->connected)
@@ -1053,6 +1127,8 @@ server_disconnect (session * sess, int sendquit, int err)
 		sprintf (tbuf, "%d", sess->server->childpid);
 		EMIT_SIGNAL (XP_TE_STOPCONNECT, sess, tbuf, NULL, NULL, NULL, 0);
 		return;
+	case 3:
+		shutup = TRUE;	/* won't print "disconnected" in channels */
 	}
 
 	server_flush_queue (serv);
@@ -1063,9 +1139,10 @@ server_disconnect (session * sess, int sendquit, int err)
 		sess = (struct session *) list->data;
 		if (sess->server == serv)
 		{
-			/* print "Disconnected" to each window using this server */
-			EMIT_SIGNAL (XP_TE_DISCON, sess, errorstring (err), NULL, NULL, NULL,
-							 0);
+			if (!shutup || sess->type == SESS_SERVER)
+				/* print "Disconnected" to each window using this server */
+				EMIT_SIGNAL (XP_TE_DISCON, sess, errorstring (err), NULL, NULL, NULL, 0);
+
 			if (!sess->channel[0] || sess->type == SESS_CHANNEL)
 				clear_channel (sess);
 		}
@@ -1081,12 +1158,21 @@ server_disconnect (session * sess, int sendquit, int err)
 	notify_cleanup ();
 }
 
+/* send a "print text" command to the parent process - MUST END IN \n! */
+
+static void
+proxy_error (int fd, char *msg)
+{
+	write (fd, "0\n", 2);
+	write (fd, msg, strlen (msg));
+}
+
 struct sock_connect
 {
 	char version;
 	char type;
-	unsigned short port;
-	unsigned long address;
+	guint16 port;
+	guint32 address;
 	char username[10];
 };
 
@@ -1095,10 +1181,10 @@ struct sock_connect
  *          1 socks traversal failed */
 
 static int
-traverse_socks (int sok, char *serverAddr, int port)
+traverse_socks (int print_fd, int sok, char *serverAddr, int port)
 {
 	struct sock_connect sc;
-	unsigned char buf[10];
+	unsigned char buf[256];
 
 	sc.version = 4;
 	sc.type = 1;
@@ -1112,6 +1198,8 @@ traverse_socks (int sok, char *serverAddr, int port)
 	if (buf[1] == 90)
 		return 0;
 
+	snprintf (buf, sizeof (buf), "SOCKS\tServer reported error %d,%d.\n", buf[0], buf[1]);
+	proxy_error (print_fd, buf);
 	return 1;
 }
 
@@ -1123,38 +1211,44 @@ struct sock5_connect1
 };
 
 static int
-traverse_socks5 (int sok, char *serverAddr, int port)
+traverse_socks5 (int print_fd, int sok, char *serverAddr, int port)
 {
 	struct sock5_connect1 sc1;
 	unsigned char *sc2;
 	unsigned int packetlen, addrlen;
 	unsigned char buf[260];
+	int auth = prefs.proxy_auth && prefs.proxy_user[0] && prefs.proxy_pass[0];
 
 	sc1.version = 5;
 	sc1.nmethods = 1;
-	if ( prefs.proxy_auth )
-	{
-		if ( !prefs.proxy_user[0] || !prefs.proxy_pass[0] )
-		{
-			return 1;
-		}
+	if (auth)
 		sc1.method = 2;  /* Username/Password Authentication (UPA) */
-	}
 	else
-	{
 		sc1.method = 0;  /* NO Authentication */
-	}
 	send (sok, (char *) &sc1, 3, 0);
 	if (recv (sok, buf, 2, 0) != 2)
-		return 1;
+		goto read_error;
 
-	if ( prefs.proxy_auth )
+	if (buf[0] != 5)
+	{
+		proxy_error (print_fd, "SOCKS\tServer is not socks version 5.\n");
+		return 1;
+	}
+
+	/* did the server say no auth required? */
+	if (buf[1] == 0)
+		auth = 0;
+
+	if (auth)
 	{
 		int len_u=0, len_p=0;
 
 		/* authentication sub-negotiation (RFC1929) */
-		if ( buf[0] != 5 || buf[1] != 2 )  /* UPA not supported by server */
+		if (buf[1] != 2)  /* UPA not supported by server */
+		{
+			proxy_error (print_fd, "SOCKS\tServer doesn't support UPA authentication.\n");
 			return 1;
+		}
 
 		memset (buf, 0, sizeof(buf));
 
@@ -1169,14 +1263,21 @@ traverse_socks5 (int sok, char *serverAddr, int port)
 
 		send (sok, buf, 3 + len_u + len_p, 0);
 		if ( recv (sok, buf, 2, 0) != 2 )
-			return 1;
+			goto read_error;
 		if ( buf[1] != 0 )
+		{
+			proxy_error (print_fd, "SOCKS\tAuthentication failed. "
+							 "Is username and password correct?\n");
 			return 1; /* UPA failed! */
+		}
 	}
 	else
 	{
-		if (buf[0] != 5 || buf[1] != 0)
+		if (buf[1] != 0)
+		{
+			proxy_error (print_fd, "SOCKS\tAuthentication required but disabled in settings.\n");
 			return 1;
+		}
 	}
 
 	addrlen = strlen (serverAddr);
@@ -1189,36 +1290,47 @@ traverse_socks5 (int sok, char *serverAddr, int port)
 	sc2[4] = (unsigned char) addrlen;	/* hostname length */
 	memcpy (sc2 + 5, serverAddr, addrlen);
 	*((unsigned short *) (sc2 + 5 + addrlen)) = htons (port);
-
 	send (sok, sc2, packetlen, 0);
 	free (sc2);
+
 	/* consume all of the reply */
 	if (recv (sok, buf, 4, 0) != 4)
+		goto read_error;
+	if (buf[0] != 5 || buf[1] != 0)
+	{
+		if (buf[1] == 2)
+			snprintf (buf, sizeof (buf), "SOCKS\tProxy refused to connect to host (not allowed).\n");
+		else
+			snprintf (buf, sizeof (buf), "SOCKS\tProxy failed to connect to host (error %d).\n", buf[1]);
+		proxy_error (print_fd, buf);
 		return 1;
-	if (buf[0] != 5 && buf[1] != 0)
-		return 1;
-	if (buf[3] == 1)
+	}
+	if (buf[3] == 1)	/* IPV4 32bit address */
 	{
 		if (recv (sok, buf, 6, 0) != 6)
-			return 1;
-	} else if (buf[3] == 4)
+			goto read_error;
+	} else if (buf[3] == 4)	/* IPV6 128bit address */
 	{
 		if (recv (sok, buf, 18, 0) != 18)
-			return 1;
-	} else if (buf[3] == 3)
+			goto read_error;
+	} else if (buf[3] == 3)	/* string, 1st byte is size */
 	{
-		if (recv (sok, buf, 1, 0) != 1)
-			return 1;
+		if (recv (sok, buf, 1, 0) != 1)	/* read the string size */
+			goto read_error;
 		packetlen = buf[0] + 2;	/* can't exceed 260 */
 		if (recv (sok, buf, packetlen, 0) != packetlen)
-			return 1;
+			goto read_error;
 	}
 
-	return 0;
+	return 0;	/* success */
+
+read_error:
+	proxy_error (print_fd, "SOCKS\tRead error from server.\n");
+	return 1;
 }
 
 static int
-traverse_wingate (int sok, char *serverAddr, int port)
+traverse_wingate (int print_fd, int sok, char *serverAddr, int port)
 {
 	char buf[128];
 
@@ -1247,7 +1359,7 @@ three_to_four (char *from, char *to)
 	to[3] = tab64 [ from[2] & 63 ];
 };
 
-static void
+void
 base64_encode (char *to, char *from, unsigned int len)
 {
 	while (len >= 3)
@@ -1304,9 +1416,9 @@ http_read_line (int print_fd, int sok, char *buf, int len)
 static int
 traverse_http (int print_fd, int sok, char *serverAddr, int port)
 {
-	char buf[256];
-	char auth_data[128];
-	char auth_data2[68];
+	char buf[512];
+	char auth_data[256];
+	char auth_data2[252];
 	int n, n2;
 
 	n = snprintf (buf, sizeof (buf), "CONNECT %s:%d HTTP/1.0\r\n",
@@ -1338,18 +1450,22 @@ traverse_http (int print_fd, int sok, char *serverAddr, int port)
 }
 
 static int
-traverse_proxy (int print_fd, int sok, char *ip, int port)
+traverse_proxy (int print_fd, int sok, char *ip, int port, struct msproxy_state_t *state, netstore *ns_proxy, int csok4, int csok6, int *csok, char bound)
 {
 	switch (prefs.proxy_type)
 	{
 	case 1:
-		return traverse_wingate (sok, ip, port);
+		return traverse_wingate (print_fd, sok, ip, port);
 	case 2:
-		return traverse_socks (sok, ip, port);
+		return traverse_socks (print_fd, sok, ip, port);
 	case 3:
-		return traverse_socks5 (sok, ip, port);
+		return traverse_socks5 (print_fd, sok, ip, port);
 	case 4:
 		return traverse_http (print_fd, sok, ip, port);
+#ifdef USE_MSPROXY
+	case 5:
+		return traverse_msproxy (sok, ip, port, state, ns_proxy, csok4, csok6, csok, bound);
+#endif
 	}
 
 	return 1;
@@ -1365,7 +1481,7 @@ server_child (server * serv)
 	netstore *ns_local;
 	int port = serv->port;
 	int error;
-	int sok;
+	int sok, psok;
 	char *hostname = serv->hostname;
 	char *real_hostname = NULL;
 	char *ip;
@@ -1373,6 +1489,7 @@ server_child (server * serv)
 	char *local_ip;
 	int connect_port;
 	char buf[512];
+	char bound = 0;
 
 	ns_server = net_store_new ();
 
@@ -1386,6 +1503,7 @@ server_child (server * serv)
 			snprintf (buf, sizeof (buf), "5\n%s\n", local_ip);
 			write (serv->childwrite, buf, strlen (buf));
 			net_bind (ns_local, serv->sok4, serv->sok6);
+			bound = 1;
 		} else
 		{
 			write (serv->childwrite, "7\n", 2);
@@ -1394,7 +1512,10 @@ server_child (server * serv)
 	}
 
 	/* first resolve where we want to connect to */
-	if (!serv->dont_use_proxy && prefs.proxy_host[0] && prefs.proxy_type > 0)
+	if (!serv->dont_use_proxy && /* blocked in serverlist? */
+		prefs.proxy_host[0] &&
+		prefs.proxy_type > 0 &&
+		prefs.proxy_use != 2)	/* proxy is NOT dcc-only */
 	{
 		snprintf (buf, sizeof (buf), "9\n%s\n", prefs.proxy_host);
 		write (serv->childwrite, buf, strlen (buf));
@@ -1407,8 +1528,8 @@ server_child (server * serv)
 		}
 		connect_port = prefs.proxy_port;
 
-		/* if using socks4, attempt to resolve ip for irc server */
-		if (prefs.proxy_type == 2)
+		/* if using socks4 or MS Proxy, attempt to resolve ip for irc server */
+		if ((prefs.proxy_type == 2) || (prefs.proxy_type == 5))
 		{
 			ns_proxy = net_store_new ();
 			proxy_ip = net_resolve (ns_proxy, hostname, port, &real_hostname);
@@ -1434,7 +1555,13 @@ server_child (server * serv)
 				 real_hostname, ip, connect_port);
 	write (serv->childwrite, buf, strlen (buf));
 
-	error = net_connect (ns_server, serv->sok4, serv->sok6, &sok);
+	if (!serv->dont_use_proxy && (prefs.proxy_type == 5))
+		error = net_connect (ns_server, serv->proxy_sok4, serv->proxy_sok6, &psok);
+	else
+	{
+		error = net_connect (ns_server, serv->sok4, serv->sok6, &sok);
+		psok = sok;
+	}
 
 	if (error != 0)
 	{
@@ -1445,10 +1572,16 @@ server_child (server * serv)
 		/* connect succeeded */
 		if (proxy_ip)
 		{
-			switch (traverse_proxy (serv->childwrite, sok, proxy_ip, port))
+			switch (traverse_proxy (serv->childwrite, psok, proxy_ip, port, &serv->msp_state, ns_proxy, serv->sok4, serv->sok6, &sok, bound))
 			{
 			case 0:	/* success */
-				snprintf (buf, sizeof (buf), "4\n%d\n", sok);	/* success */
+#ifdef USE_MSPROXY
+				if (!serv->dont_use_proxy && (prefs.proxy_type == 5))
+					snprintf (buf, sizeof (buf), "4\n%d %d %d %d %d\n", sok, psok, serv->msp_state.clientid, serv->msp_state.serverid,
+						serv->msp_state.seq_sent);
+				else
+#endif
+					snprintf (buf, sizeof (buf), "4\n%d\n", sok);	/* success */
 				write (serv->childwrite, buf, strlen (buf));
 				break;
 			case 1:	/* socks traversal failed */
@@ -1489,7 +1622,7 @@ static void
 server_connect (server *serv, char *hostname, int port, int no_login)
 {
 	int pid, read_des[2];
-	session *sess;
+	session *sess = serv->server_session;
 
 #ifdef USE_OPENSSL
 	if (!ctx && serv->use_ssl)
@@ -1505,7 +1638,7 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	if (!hostname[0])
 		return;
 
-	if (port < 0 || port >= 65536)
+	if (port < 0)
 	{
 		/* use default port for this server type */
 		port = 6667;
@@ -1514,8 +1647,7 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 			port = 9999;
 #endif
 	}
-
-	sess = serv->server_session;
+	port &= 0xffff;	/* wrap around */
 
 	if (serv->connected || serv->connecting || serv->recondelay_tag)
 		server_disconnect (sess, TRUE, -1);
@@ -1525,13 +1657,28 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	EMIT_SIGNAL (XP_TE_SERVERLOOKUP, sess, hostname, NULL, NULL, NULL, 0);
 
 	safe_strcpy (serv->servername, hostname, sizeof (serv->servername));
-	safe_strcpy (serv->hostname, hostname, sizeof (serv->hostname));
+	/* overlap illegal in strncpy */
+	if (hostname != serv->hostname)
+		safe_strcpy (serv->hostname, hostname, sizeof (serv->hostname));
+
+#ifdef USE_OPENSSL
+	if (serv->use_ssl)
+	{
+		char cert_file[256];
+
+		snprintf (cert_file, sizeof (cert_file), "%s/%s.pem",
+					 get_xdir_fs (), server_get_network (serv, TRUE));
+		if (SSL_CTX_use_certificate_file (ctx, cert_file, SSL_FILETYPE_PEM) == 1)
+			SSL_CTX_use_PrivateKey_file (ctx, cert_file, SSL_FILETYPE_PEM);
+	}
+#endif
 
 	server_set_defaults (serv);
 	serv->connecting = TRUE;
 	serv->port = port;
 	serv->no_login = no_login;
 
+	fe_server_event (serv, FE_SE_CONNECTING, 0);
 	fe_set_away (serv);
 	server_flush_queue (serv);
 
@@ -1550,12 +1697,25 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 
 	/* create both sockets now, drop one later */
 	net_sockets (&serv->sok4, &serv->sok6);
+#ifdef USE_MSPROXY
+	/* In case of MS Proxy we have a separate UDP control connection */
+	if (!serv->dont_use_proxy && (prefs.proxy_type == 5))
+		udp_sockets (&serv->proxy_sok4, &serv->proxy_sok6);
+	else
+#endif
+	{
+		serv->proxy_sok4 = -1;
+		serv->proxy_sok6 = -1;
+	}
 
 #ifdef WIN32
 	CloseHandle (CreateThread (NULL, 0,
 										(LPTHREAD_START_ROUTINE)server_child,
 										serv, 0, (DWORD *)&pid));
 #else
+#ifdef LOOKUPD
+	rand();	/* CL: net_resolve calls rand() when LOOKUPD is set, so prepare a different seed for each child. This method giver a bigger variation in seed values than calling srand(time(0)) in the child itself. */
+#endif
 	switch (pid = fork ())
 	{
 	case -1:
@@ -1596,6 +1756,7 @@ server_set_encoding (server *serv, char *new_encoding)
 		/* can be left as NULL to indicate system encoding */
 		serv->encoding = NULL;
 		serv->using_cp1255 = FALSE;
+		serv->using_irc = FALSE;
 	}
 
 	if (new_encoding)
@@ -1607,10 +1768,12 @@ server_set_encoding (server *serv, char *new_encoding)
 		if (space)
 			space[0] = 0;
 
-		/* server_inline() uses this flag */
+		/* server_inline() uses these flags */
 		if (!strcasecmp (serv->encoding, "CP1255") ||
 			 !strcasecmp (serv->encoding, "WINDOWS-1255"))
 			serv->using_cp1255 = TRUE;
+		else if (!strcasecmp (serv->encoding, "IRC"))
+			serv->using_irc = TRUE;
 	}
 }
 
@@ -1662,7 +1825,7 @@ server_set_defaults (server *serv)
 	serv->nick_modes = strdup ("ohv");
 
 	serv->nickcount = 1;
-	serv->nickservtype = 0;
+	serv->nickservtype = 1;
 	serv->end_of_motd = FALSE;
 	serv->is_away = FALSE;
 	serv->supports_watch = FALSE;
