@@ -81,9 +81,6 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define USERNAMELEN 10
 #define HIDDEN_CHAR	8			/* invisible character for xtext */
 
-#define safe_strcpy(dest,src,len)	{strncpy((dest),(src),(len)); \
-												(dest)[len-1] = 0;}
-
 #if defined(ENABLE_NLS) && !defined(_)
 #  include <libintl.h>
 #  define _(x) gettext(x)
@@ -262,6 +259,10 @@ struct xchatprefs
 	unsigned int auto_indent;
 	unsigned int wordwrap;
 	unsigned int gui_input_spell;
+	unsigned int gui_tray;
+	unsigned int gui_tray_blink;
+	unsigned int gui_tray_flags;
+	unsigned int gui_ulist_left;
 	unsigned int throttle;
 	unsigned int topicbar;
 	unsigned int hideuserlist;
@@ -342,6 +343,7 @@ typedef struct session
 	/* these are in the bottom-right menu */
 	unsigned int hide_join_part:1;	/* hide join & part messages? */
 	unsigned int beep:1;				/* beep enabled? */
+	unsigned int tray:1;				/* tray flash for this chan? */
 	unsigned int color_paste:1;
 	int done_away_check:1;	/* done checking for away status changes */
 	unsigned int lastlog_regexp:1;	/* this is a lastlog and using regexp */
@@ -389,7 +391,7 @@ typedef struct server
 	void (*p_action)(struct server *, char *channel, char *act);
 	void (*p_notice)(struct server *, char *channel, char *text);
 	void (*p_topic)(struct server *, char *channel, char *topic);
-	void (*p_list_channels)(struct server *, char *arg);
+	void (*p_list_channels)(struct server *, char *arg, int min_users);
 	void (*p_change_nick)(struct server *, char *new_nick);
 	void (*p_names)(struct server *, char *channel);
 	void (*p_ping)(struct server *, char *to, char *timestring);
@@ -409,6 +411,8 @@ typedef struct server
 #ifdef USE_OPENSSL
 	SSL *ssl;
 	int ssl_do_connect_tag;
+#else
+	void *ssl;
 #endif
 	int childread;
 	int childwrite;
