@@ -1,7 +1,7 @@
 /*
  * fe-gnome.c - main frontend implementation
  *
- * Copyright (C) 2004-2006 xchat-gnome team
+ * Copyright (C) 2004-2007 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,10 +38,12 @@
 #include "status-bar.h"
 #include "topic-label.h"
 #include "connect-dialog.h"
+#include "text-entry.h"
+#include "find-bar.h"
 
 #include "palette.h"
 #include "preferences-page-plugins.h"
-#include "channel-list.h"
+#include "channel-list-window.h"
 #include "util.h"
 #include "plugins.h"
 #include "../common/xchat.h"
@@ -51,6 +53,7 @@
 #include "../common/fe.h"
 #include "../common/util.h"
 #include "../common/cfgfiles.h"
+#include "../common/plugin.h"
 
 static gboolean  opt_version   = FALSE;
 static gboolean  opt_noplugins = FALSE;
@@ -61,7 +64,7 @@ static GOptionEntry entries[] = {
 	{"no-auto",    'a', 0, G_OPTION_ARG_NONE,     &arg_dont_autoconnect, N_("Don't auto-connect to servers"),                   NULL},
 	{"no-plugins", 'n', 0, G_OPTION_ARG_NONE,     &opt_noplugins,        N_("Don't auto-load plugins"),                         NULL},
 	{"url",        'u', 0, G_OPTION_ARG_STRING,   &arg_url,              N_("Open an irc:// url"),                              "irc://server:port/channel"},
-	{"existing",   'e', 0, G_OPTION_ARG_NONE,     &arg_existing,         N_("Open URL in an existing xchat-gnome instance"),    NULL},
+	{"existing",   'e', 0, G_OPTION_ARG_NONE,     &arg_existing,         N_("Open URL in an existing XChat-GNOME instance"),    NULL},
 	{"version",    'v', 0, G_OPTION_ARG_NONE,     &opt_version,          N_("Show version information"),                        NULL},
 	{ NULL }
 };
@@ -222,26 +225,21 @@ fe_new_window (struct session *sess, int focus)
 {
 	static gboolean loaded = FALSE;
 
-	if (focus) {
-		gui.current_session = sess;
-		topic_label_set_topic (TOPIC_LABEL (gui.topic_label), sess, sess->topic);
-	}
-
 	conversation_panel_add_session (CONVERSATION_PANEL (gui.conversation_panel), sess, (gboolean) focus);
-	if (sess->type == SESS_SERVER) {
-		navigation_tree_create_new_network_entry (gui.server_tree, sess);
-	} else if (sess->type == SESS_CHANNEL) {
-		navigation_tree_create_new_channel_entry (gui.server_tree, sess, (gboolean) focus);
-	} else if (sess->type == SESS_DIALOG) {
-		struct User *user = NULL;
 
-		user = userlist_find_global (sess->server, sess->channel);
-
-		navigation_tree_create_new_channel_entry (gui.server_tree, sess, (gboolean) focus);
-		if (user) {
-			topic_label_set_topic (TOPIC_LABEL (gui.topic_label), sess, user->hostname);
-		}
+	switch (sess->type) {
+	case SESS_SERVER:
+		navigation_model_add_server (gui.tree_model, sess);
+		break;
+	case SESS_CHANNEL:
+	case SESS_DIALOG:
+		navigation_model_add_channel (gui.tree_model, sess);
 	}
+
+	if (focus) {
+		navigation_tree_select_session (gui.server_tree, sess);
+	}
+
 #ifdef USE_PLUGIN
 	if (!(opt_noplugins || loaded)) {
 		loaded = TRUE;
@@ -393,12 +391,25 @@ fe_text_clear (struct session *sess)
 void
 fe_close_window (struct session *sess)
 {
-	if (!gui.quit) {
-		if (sess->type == SESS_CHANNEL) {
-			navigation_tree_remove_channel (gui.server_tree, sess);
-		} else {
-			navigation_tree_remove_server (gui.server_tree, sess);
-		}
+	/*
+	 * There's really no point in doing all of this if the user is
+	 * quitting the app.  It makes it slow (as they watch individual
+	 * channels and servers disappear), and the OS is about to free
+	 * everything much more efficiently than we ever could.
+	 *
+	 * If we ever choose to run on Windows ME, this could be a problem :)
+	 */
+	if (gui.quit) {
+		session_free (sess);
+		return;
+	}
+
+	navigation_tree_remove_session (gui.server_tree, sess);
+	conversation_panel_remove_session (CONVERSATION_PANEL (gui.conversation_panel), sess);
+	topic_label_remove_session (TOPIC_LABEL (gui.topic_label), sess);
+	text_entry_remove_session (TEXT_ENTRY (gui.text_entry), sess);
+	if (sess->type == SESS_SERVER) {
+		status_bar_remove_server (STATUS_BAR (gui.status_bar), sess->server);
 	}
 
 	session_free (sess);
@@ -546,14 +557,20 @@ fe_dcc_open_chat_win (int passive)
 void
 fe_clear_channel (struct session *sess)
 {
-	navigation_model_set_disconn (gui.tree_model, sess);
+	navigation_model_set_disconnected (gui.tree_model, sess);
 }
 
 void
 fe_session_callback (struct session *sess)
 {
-	/* this frees things */
-	/* FIXME: implement */
+	if (sess->type == SESS_SERVER) {
+		status_bar_remove_server (STATUS_BAR (gui.status_bar), sess->server);
+	}
+
+	conversation_panel_remove_session (CONVERSATION_PANEL (gui.conversation_panel), sess);
+	topic_label_remove_session (TOPIC_LABEL (gui.topic_label), sess);
+	text_entry_remove_session (TEXT_ENTRY (gui.text_entry), sess);
+	userlist_erase (u, sess);
 }
 
 void
@@ -595,7 +612,7 @@ fe_dcc_send_filereq (struct session *sess, char *nick, int maxcps, int passive)
 void
 fe_set_channel (struct session *sess)
 {
-    navigation_tree_set_channel_name (gui.server_tree, sess);
+	navigation_model_update (gui.tree_model, sess);
 }
 
 void
@@ -620,7 +637,7 @@ fe_set_nonchannel (struct session *sess, int state)
 void
 fe_set_nick (struct server *serv, char *newnick)
 {
-	set_nickname (serv, newnick);
+	set_nickname_label (serv, newnick);
 }
 
 void
@@ -688,30 +705,30 @@ fe_serverlist_open (session * sess)
 }
 
 void
-fe_ctrl_gui (session * sess, int action, int arg)
+fe_ctrl_gui (session * sess, fe_gui_action action, int arg)
 {
 	switch (action) {
-	case 0:
+	case FE_GUI_HIDE:
 		gtk_widget_hide (gui.main_window);
 		break;
 
-	case 1:
+	case FE_GUI_SHOW:
 		gtk_widget_show (gui.main_window);
 		gtk_window_present (GTK_WINDOW (gui.main_window));
 		break;
 
-	case 2:
-		break;
-
-	case 3:
-		break;
-
-	case 4:
-		break;
-
-	case 5:
+	case FE_GUI_ICONIFY:
 		gtk_window_iconify (GTK_WINDOW (gui.main_window));
 		break;
+
+	case FE_GUI_FOCUS:
+	case FE_GUI_FLASH:
+	case FE_GUI_MENU:
+	case FE_GUI_ATTACH:
+	case FE_GUI_APPLY:
+	case FE_GUI_COLOR:
+		/* These are unhandled for now */
+		break;
 	}
 }
 
@@ -956,9 +973,21 @@ fe_server_event (server *serv, int type, int arg)
 			case FE_SE_LOGGEDIN:
 				if (arg == 0) {
 					/* No auto-join channels */
-					create_channel_list (sess);
-					break;
+					GConfClient *client;
+					gboolean popup_channel_list;
+
+					client = gconf_client_get_default ();
+					popup_channel_list = gconf_client_get_bool (client,
+							"/apps/xchat/channel_list/auto_popup",  NULL);
+					g_object_unref (client);
+
+					if (popup_channel_list)
+						create_channel_list_window (sess, FALSE);
 				}
+				break;
+			case FE_SE_DISCONNECT:
+				navigation_model_set_disconnected (gui.tree_model, sess);
+				break;
 			}
 		}
 		list = list->next;
@@ -1055,3 +1084,49 @@ not_autoconnect (void)
 
 	return TRUE;
 }
+
+void
+fe_set_current (session *sess)
+{
+	// If find bar is open, hide it
+	find_bar_close (FIND_BAR (gui.find_bar));
+
+	gui.current_session = sess;
+
+	// Notify parts of the UI that the current session has changed
+	conversation_panel_set_current (CONVERSATION_PANEL (gui.conversation_panel), sess);
+	topic_label_set_current (TOPIC_LABEL (gui.topic_label), sess);
+	text_entry_set_current (TEXT_ENTRY (gui.text_entry), sess);
+	status_bar_set_current (STATUS_BAR (gui.status_bar), sess->server);
+	navigation_model_set_current (gui.tree_model, sess);
+
+	// Change the window name
+	if (sess->server->network == NULL) {
+		rename_main_window (NULL, sess->channel);
+	} else {
+		ircnet *net = sess->server->network;
+		rename_main_window (net->name, sess->channel);
+	}
+
+	// Set nickname button
+	set_nickname_label (sess->server, NULL);
+
+	// Set the label of the user list button
+	userlist_set_user_button (u, sess);
+	gtk_widget_set_sensitive (GTK_WIDGET (gui.userlist_toggle), sess->type == SESS_CHANNEL);
+
+	// FIXME: Userlist should be more encapsulated
+	gtk_tree_view_set_model (GTK_TREE_VIEW (gui.userlist), GTK_TREE_MODEL (userlist_get_store (u, sess)));
+
+	// Emit "focus tab" event for plugins that rely on it
+	plugin_emit_dummy_print (sess, "Focus Tab");
+
+	gtk_widget_grab_focus (GTK_WIDGET (gui.text_entry));
+}
+
+/* Tray stubs -- xchat-gnome bundles this as a plugin */
+void fe_tray_set_flash (const char *filename1, const char *filename2, int timeout) {}
+void fe_tray_set_file (const char *filename) {}
+void fe_tray_set_icon (int icon) {}
+void fe_tray_set_tooltip (const char *text) {}
+void fe_tray_set_balloon (const char *title, const char *text) {}
